package exampler.fitlibrary;

import fit.Parse;
import fitlibrary.parse.Cell;
import fitlibrary.parse.Row;
import fitlibrary.parse.Table;
import junit.framework.TestCase;

public class TableParseTests extends TestCase {
    private String[][] bigTableSource =
            new String[][]{ { "0,0", "0,1" },
                            { "1,0", "1,1" },
                            { "2,0", "2,1" } };

    public void testRowIndexing() {
        TableParse table = TableParse.fromArray(bigTableSource);
        assertEquals("0,0", table.row(0).parts.text());
        assertEquals("2,0", table.row(2).parts.text());
    }


    public void testCreatingAnEmptyTable() {
        String[][] source = {};
        Parse table = TableParse.fromArray(source);
        assertNull(table.more);
        assertNull(table.parts);
        assertEquals("<table>", table.tag);
        assertEquals("", table.text());
    }

    public void testCreatingATableWithAnEmptyRow() {
        String[][] source = { {} };
        Parse parseTable = TableParse.fromArray(source);
        Table table = new Table(parseTable);
        assertEquals(1, table.size());
        Row row = table.row(0);
        assertEquals("<tr>", row.parse.tag);
        assertNull(row.parse.parts);
        assertNull(row.parse.more);
    }

    public void testCreatingATableWithRowContents() {
        String[][] source = { { "0,0" } };
        Parse parseTable = TableParse.fromArray(source);
        Table table = new Table(parseTable);
        Row row = table.row(0);
        assertEquals(1, row.size());
        Cell cell = row.cell(0);
        assertEquals("<td>", cell.parse.tag);
        assertEquals("0,0", cell.parse.text());
        assertNull(cell.parse.parts);
        assertNull(cell.parse.more);

    }


    public void testCreatingACompleteTable() {
        Parse parseTable = TableParse.fromArray(bigTableSource);
        Table table = new Table(parseTable);
        assertEquals(3, table.size());
        Row firstRow = table.row(0);
        assertEquals(2, firstRow.size());
        Row lastRow = table.row(2);
        assertEquals(2, lastRow.size());

        assertEquals("0,0", firstRow.cell(0).text());
        assertEquals("0,1", firstRow.cell(1).text());

        assertEquals("2,0", lastRow.cell(0).text());
        assertEquals("2,1", lastRow.cell(1).text());
    }



    public void testAddingADerivedTableInsertsAfter() {
        TableParse inFront = new TableParse();
        TableParse behind = new TableParse();
        inFront.more = behind;

        TableParse inserted = new TableParse();

        inserted.insertAfter(new Table(inFront));

        assertSame(inFront.more, inserted);
        assertSame(inserted.more, behind);
    }
    public void testAddingADerivedTableShiftsOnlyTrailersDown() {
        // Fit invariant: Only the last table should ever have a trailer.
        TableParse inFront = new TableParse();
        inFront.leader = "infront leader";
        inFront.trailer = "infront trailer";

        TableParse inserted = new TableParse();
        inserted.insertAfter(new Table(inFront));

        assertEquals("infront leader", inFront.leader);
        assertEquals("", inFront.trailer);
        assertEquals("infront trailer", inserted.trailer);
    }

}
